#include <Servo.h>
#include <wiring_private.h>
#include <pins_arduino.h>

/***   Global variables   ***/
int _white=800;
int _black=30;
const uint16_t THRILLER[] = {196,187,220,187,262,187,294,187,220,187,0,93,220,93,0,187,220,93};
volatile uint16_t* _current_melody_ptr;
volatile int _melody_pin=0;
volatile int _melody_counter=0;
volatile int _melody_length=0;
volatile uint8_t _melody_status=-1;
volatile unsigned long _melody_next_time=0;
unsigned long _currentTime;
Servo _servo3;
Servo _servo5;
Servo _servo6;
const uint16_t LOVE[] = {392,187,392,187,392,187,330,187,392,750};

/***   Function declaration   ***/
void LEDMatrix_init(int cs, int din, int clk);
void writeRow(int cs, int din, int clk, int row, int data);
void maxAll (int cs, int din, int clk, int reg, int col);
void putByte (int din, int clk, int data);
void expression(int cs, int din, int clk, int col1, int col2, int col3, int col4, int col5, int col6, int col7, int col8);
void prepareMelody(int pin,const uint16_t* melody, int length);
void playMelodyInterrupt();
int follow(int light, int gain);
void move_base_5_3(int speed, int advance, int turn);
unsigned long _pulseIn(uint8_t pin, uint8_t state, unsigned long timeout);
long US_init(int trigger_pin, int echo_pin);
long distance(int trigger_pin, int echo_pin);

/***   Tasks declaration   ***/
SIGNAL (TIMER0_COMPA_vect){
playMelodyInterrupt();
}

/***   ISR function declaration   ***/

void setup()
{
  pinMode(10,OUTPUT);
  pinMode(12,OUTPUT);
  pinMode(11,OUTPUT);

  maxAll(10,12,11,11,7);
  maxAll(10,12,11,9,0);
  maxAll(10,12,11,12,1);
  maxAll(10,12,11,15,0);
  int i=0;
  for (i = 1; i <= 8; i++)
    maxAll(10,12,11,i,0);
  maxAll(10,12,11,10,15);

    _white=40;
  _black=730;
  expression(10,12,11,0,64,64,126,127,67,70,0);
  delay(2000);

  OCR0A=0xAF;
  TIMSK0 |= _BV(OCIE0A);

  _servo3.attach(3);
  _servo5.attach(5);
  _servo6.attach(6);
  pinMode( (1) , INPUT );
  pinMode( (0) , OUTPUT );
}


void loop()
{
    if (distance((0),(1))<(12)){
      expression(10,12,11,0,0,0,24,24,0,0,0);
      prepareMelody(2,THRILLER,sizeof(THRILLER)/(2*sizeof(uint16_t)));
      _servo3.detach();
      _servo5.detach();
      _servo6.write(0);
      delay(1000);
      _servo6.write(50);
      delay(1000);

    }
    else
    {
      _servo3.attach(3);
      _servo5.attach(5);
      expression(10,12,11,48,24,12,12,12,12,24,48);
      prepareMelody(2,LOVE,sizeof(LOVE)/(2*sizeof(uint16_t)));
      move_base_5_3(12,30,follow(analogRead((A0)),10));
    }

}

/***   Function definition   ***/
void LEDMatrix_init(int cs, int din, int clk) {
maxAll(cs,din,clk,11,7);
maxAll(cs,din,clk,9,0);
maxAll(cs,din,clk,12,1);
maxAll(cs,din,clk,15,0);
int i=0;
for (i = 1; i <= 8; i++) {
maxAll(cs,din,clk,i, 0);
}
maxAll(cs,din,clk,10,15);
}
void writeRow(int cs, int din, int clk, int row, int data) {
digitalWrite(cs,LOW);
putByte(din,clk,row);
putByte(din,clk,data);
digitalWrite(cs,LOW);
digitalWrite(cs,HIGH);
}
void maxAll (int cs, int din, int clk, int reg, int col) {
digitalWrite(cs,LOW);
putByte(din,clk,reg);
putByte(din,clk,col);
digitalWrite(cs,LOW);
digitalWrite(cs,HIGH);
}
void putByte (int din, int clk, int data) {
byte i = 8;
byte mask;
while(i > 0) {mask = 0x01 << (i - 1);
    digitalWrite(clk,LOW);
    if (data & mask)
{
      digitalWrite(din,HIGH);
    }else{
      digitalWrite(din,LOW);
    }
    digitalWrite(clk,HIGH);
    --i;
}
}
void expression(int cs, int din, int clk, int col1, int col2, int col3, int col4, int col5, int col6, int col7, int col8) {
writeRow(cs,din,clk,1, col1);
writeRow(cs,din,clk,2, col2);
writeRow(cs,din,clk,3, col3);
writeRow(cs,din,clk,4, col4);
writeRow(cs,din,clk,5, col5);
writeRow(cs,din,clk,6, col6);
writeRow(cs,din,clk,7, col7);
writeRow(cs,din,clk,8, col8);
}
void prepareMelody(int pin,const uint16_t* melody, int length)
{
  if (_melody_status==1)
    return;
  _melody_pin=pin;
  _current_melody_ptr=(uint16_t*)melody;
  _melody_length=length;
  _melody_counter=0;
  noTone(_melody_pin);
  _melody_next_time=millis()+2;
  _melody_status=1;
}
void playMelodyInterrupt()
{
  unsigned int _melody_note=0;
  unsigned long _melody_duration=0;
  unsigned long currentTime=millis();
   if (_melody_status==0)
 {
  noTone(_melody_pin);
  _melody_status=-1;
 }
 else if (_melody_status==1){
  if (_melody_counter>_melody_length)
  {
    _melody_status=0;
  }
  if (currentTime>=_melody_next_time)
  {
    _melody_note=*_current_melody_ptr++;
    _melody_duration=*_current_melody_ptr++;
    tone(_melody_pin,_melody_note,_melody_duration);
    _melody_next_time=currentTime+_melody_duration;
    _melody_counter++;
  }
 }
}
int follow(int light, int gain)
{
  return (int)(((float)(gain)/100.0)*((float)((_white-_black)/2+_black)-(float)(light)));
}
void move_base_5_3(int speed, int advance, int turn)
{
_servo5.write(90+(int)(65.0*(float)(speed*(advance-turn))/10000.0));
_servo3.write(90-(int)(65.0*(float)(speed*(advance+turn))/10000.0));
};
unsigned long _pulseIn(uint8_t pin, uint8_t state, unsigned long timeout)
{
  uint8_t bit = digitalPinToBitMask(pin);
  uint8_t port = digitalPinToPort(pin);
  uint8_t stateMask = (state ? bit : 0);
  unsigned long width = 0;
  unsigned long numloops = 0;
  unsigned long maxloops = microsecondsToClockCycles(timeout) / 16;
  while ((*portInputRegister(port) & bit) == stateMask)
    if (numloops++ == maxloops)
      return 0;
  while ((*portInputRegister(port) & bit) != stateMask)
    if (numloops++ == maxloops)
      return 0;
  while ((*portInputRegister(port) & bit) == stateMask) {
    if (numloops++ == maxloops)
      return 0;
    width++;
  }
  return clockCyclesToMicroseconds(width * 21 + 16);
}
long US_init(int trigger_pin, int echo_pin)
{
  digitalWrite(trigger_pin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigger_pin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigger_pin, LOW);
  long microseconds = _pulseIn(echo_pin ,HIGH,100000);
  return microseconds;
}
long distance(int trigger_pin, int echo_pin)
{
  long microseconds = US_init(trigger_pin, echo_pin);
  long distance;
  distance = microseconds/29/2;
  if (distance == 0){
    distance = 999;
  }
  return distance;
}
